// Registro.cpp: implementation of the CRegistro class.
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Registro.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <winreg.h>
#include <stdio.h>
#include <iostream.h>
#include <mbstring.h>

CRegistro* CRegistro::p_Reg=0;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRegistro::CRegistro()
{
	p_Reg = this;
}

CRegistro::~CRegistro()
{

}

/**************************************************************************
FUNCION: openReadSubkey()
DESCRIPCION: Esta funcin abre una subclave del registro de windows apuntada
			por psubkey (con permiso para consultar los datos de la subclave) 
			que se encuentra dentro de la clave identificada por handleKey.
			En phandleResult queda el identificador de la subclave abierta.
INPUT:
  handleKey		= identificador de una clave abierta o uno de los identificadores
				  predefinidos como HKEY_CLASSES_ROOT
  psubkey		= apuntador a la subclave que se abrir
  phandleResult = direccin donde se almacenar el identificador de la 
				  subclave abierta
OUTPUT:
  0 si la operacion resulto exitosa
  1 si ocurre algn error
***************************************************************************/

USHORT CRegistro::openReadSubkey(HKEY handleKey, char *psubkey, PHKEY phandleResult)
{
	exito=RegOpenKeyEx(	handleKey,			// handle to open key
						psubkey,			// address of name of subkey to open
						0,					// reserved
						KEY_QUERY_VALUE,	// security access mask. Permission to query subkey data. 
						phandleResult);		// address of handle to open key

	if (exito!=0) 
		return 1;
	
	return 0;
}

/**************************************************************************
FUNCION: openSubkey()
DESCRIPCION: Esta funcin abre o crea una subclave(apuntada por psubkey) del 
			registro de windows dentro de la clave identificada por handleKey
			(con permisos de acceso total sobre la subclave).
			En phandleResult queda el identificador de la subclave abierta.
INPUT:
  handleKey		= identificador de una clave abierta o uno de los identificadores
				  predefinidos como HKEY_CLASSES_ROOT
  psubkey		= apuntador a la subclave que se abrir
  phandleResult = direccin donde se almacenar el identificador de la 
				  subclave abierta
OUTPUT:
  0 si la operacion resulto exitosa
  1 si ocurre algn error

NOTA:
	Si la subclave no existe ser creada.
***************************************************************************/

USHORT CWR_Reg::openSubkey(HKEY handleKey, char *psubkey, PHKEY phandleResult)
{
	nameClass	= "Telefonia IP";
	presult		= &result;

	exito = RegCreateKeyEx(handleKey,			// handle to an open key  
						psubkey,				// address of subkey name
						0,						// reserved
						nameClass,				// address of class string
												// apunta a una cadena que identifica
												// el tipo de objeto de la subclave
						REG_OPTION_NON_VOLATILE,// special options flag
						KEY_ALL_ACCESS,			// permiso de acceso total. Desired security access
						NULL,					// the handle cannot be inherited
												// especifica que el identificador no
												// puede ser heredado por procesos hijos
												// address of key security structure
						phandleResult,			// address of buffer for opened handle
												// receives a handle to the opened or created key
						presult);				// direccin del buffer que contendr el 
												// valor REG_OPENED_EXISTING_KEY si la subclave
												// exista y fue abierta o REG_CREATED_NEW_KEY
												// si la clave fue creada.
												// address of disposition value buffer

	if (exito!=0) 
		return 1;
	
	return 0;
}

/**************************************************************************
FUNCION: readValueReg()
DESCRIPCION: Esta funcin permite consultar el contenido de un valor
			 perteneciente a una subclave del registro.
IN:
  handleKey		= identificador de una clave abierta o uno de los identificadores
				  predefinidos como HKEY_CLASSES_ROOT
  pnameValue	= apuntador al nombre del Valor que se desea consultar
  pdata			= direccin donde se almacenarn los datos leidos

OUT:
  0 si la operacion resulto exitosa
  1 si ocurre algn error
***************************************************************************/
 
USHORT CRegistro::readValueReg(HKEY handleKey, CString pnameValue, UCHAR *pdata)
{
	sizeData		= 300;			// cantidad de datos que sern leidos (en bytes)
	ptypeData		= &typeData;	// dirrecin del buffer para almacenar el tipo de dato leido
	psizeData		= &sizeData;	// address of data buffer size 

	exito	= RegQueryValueEx(handleKey,	// handle to key to query
					pnameValue,				// address of name of value to query
					0,						// reserved
					ptypeData,				// address of buffer for value type
					pdata,					// address of data buffer
					psizeData);				// address of data buffer size  

	if (exito!=0) 
		return 1;

	return 0;
}


/****************************************************************************************
FUNCION: setValueReg()
DESCRIPCION: Esta funcin permite modificar el contenido de un valor
			 perteneciente a una subclave del registro.
IN:
  handleKey		= identificador de una clave abierta o uno de los identificadores
				  predefinidos como HKEY_CLASSES_ROOT
  pnameValue	= apuntador al nombre del Valor que se desea modificar
  pdata			= apuntador a la cadena de caracteres que contiene los datos
				  que se desean almacenar.

OUT:
  0 si la operacion resulto exitosa
  1 si ocurre algn error

NOTA: 
	
	Si no existe un valor con el mismo nombre apuntado por pnameValue, este sera creado.
*****************************************************************************************/

USHORT CWR_Reg::setValueReg(HKEY handleKey, CString pnameValue, UCHAR *pdata)
{
	sizeData	= _mbslen(pdata);	// tamao de la cadena de caracteres que ser almacenada
									// _mbslen recognizes multibyte-character sequences according 
									// to the multibyte code page currently in use
	
	exito = RegSetValueEx(handleKey,// handle to key to set value for
						pnameValue,	// name of the value to set
						0,			// Reserved
						REG_SZ,		// Bandera para el tipo de dato del valor
									// En este caso una cadena con terminador nulo.
						pdata,		// address of value data
						sizeData);	// size of value data

	if (exito!=0) 
		return 1;
	
	return 0;
}

/*****************************************************************************************
* Funcin que obtiene el nmero de subkey de la llave pasada por handleKey
*****************************************************************************************/
DWORD CWR_Reg::InfoReg(HKEY handleKey)
{
	RegQueryInfoKey( 
		handleKey,            // handle to key to query
		NULL,				  // address of buffer for class string
		NULL,				  // address of size of class string buffer
		NULL,				  // reserved
		&numberSubKeys,		  // address of buffer for number of subkeys
		NULL,				  // address of buffer for longest subkey  name length
		NULL,				  // address of buffer for longest class string length
		NULL,				  // address of buffer for number of value  entries
		&lpcbMaxValueNameLen, // address of buffer for longest  value name length
		NULL,				  // address of buffer for longest value data length  
		NULL,				  // address of buffer for security  descriptor length
		NULL				  // address of buffer for last write time
		);
	
	return numberSubKeys;
}

/*****************************************************************************************
* Nombre de los subregistros que pertenecen a la llave apuntada por handleKey
*****************************************************************************************/
USHORT CWR_Reg::EnumReg(HKEY handleKey, DWORD numSubKey)
{
	char lpName[30];
		
	for (int i=0; i<30; i++)
		lpName[i]=NULL;

	exito=RegEnumKey(  
		handleKey,      // handle to key to query
		numSubKey,		// index of subkey to query
		lpName,			// address of buffer for subkey name
		30			// size of subkey buffer
		);

	Aux=lpName;
		
	if (exito!=0)
		return 1;

	return 0;
}

/*****************************************************************************************
* Obtiene los nombres de las subllaves apuntada por hKey
*****************************************************************************************/
USHORT CWR_Reg::EnumValue(HKEY hKey, DWORD dwIndex)
{
	char lpName[30];
	DWORD lpcbValueName;
	//BYTE lpData;
	//DWORD lpcbData1;

	char UNO[200];
	itoa((int)hKey,UNO,10);
		
	for (int i=0; i<30; i++)
		lpName[i]=NULL;
	
	exito=RegEnumValue(
						hKey,             // handle to key to query
						dwIndex,          // index of value to query
						lpName,			  // address of buffer for value string
						&lpcbValueName,	  // address for size of value buffer
						NULL,             // reserved
						NULL,             // address of buffer for type code
						NULL,             // address of buffer for value data
						NULL              // address for size of data buffer
						);
//Para obtener el valor de las varibles se utilizan los dos ultimos parmetros
	//&lpData,          // address of buffer for value data
	//&lpcbData1       // address for size of data buffer);

	
	Aux=lpName;
	
	if (exito!=0)
		return 1;

	return 0;
}

/*****************************************************************************************
* Borra la llave del registro apuntada por handleKey
*****************************************************************************************/
USHORT CWR_Reg::DeleteKey(HKEY handleKey, LPCTSTR lpSubKey)
{
	exito=RegDeleteKey( 
						handleKey,  // handle to open key
						lpSubKey	// address of name of subkey to delete
						);

	if (exito!=0)
		return 1;

	return 0;
}

/*****************************************************************************************
* Cierra la llave del registro apuntada por handleKey
*****************************************************************************************/
USHORT CWR_Reg::CloseKey(HKEY handleKey)
{
	exito=RegCloseKey( handleKey);

	if (exito!=0)
		return 1;

	return 0;
}

/*****************************************************************************************
* Borra la llave del registro apuntada por handleKey
*****************************************************************************************/
USHORT CWR_Reg::ArchivoReg(HKEY handleKey, LPCTSTR lpFile)
{
	exito=RegSaveKey(  
						handleKey,    // handle to key where save begins
						lpFile,  // address of filename to save to
						NULL	 // address of security structure
						);

	if (exito!=0)
		return 1;

	return 0;
}